FUNCTION read_specpr_header,specprfile,irecord,title,record_header,DESCRIPT_STR=descript_str,$
 NUM_RECS=num_recs,NO_PRINT=no_print,DO_PS=do_ps,WAVES=waves
;-------------------------------------------------------------------------------
;
;  RETURNS 0 = data record
;          1 = text record
;         -1 = if the requested record is beyond the end of file
;         -2 = if the file could not be open/read
;
; *********** NEED TO ADD CODE TO CHECK TO SEE IF REQUESTED RECORD IS A CONTINUATION RECORD
;
; NOTES:
;  Program returns iscta and isctb as seconds in floating point format (scale factor removed).
;  Program returns jdatea and jdateb as Julian Day in floating point format (scale factor removed).
;  COMMENTS: THERE IS UNEQUAL TREATMENT OF SCALED PARAMETERS (e.g jdatea is expected to be input
;            as an floating-point unscaled value, however, isdec is expected to be a scaled 4-byte integer).
;
;-------------------------------------------------------------------------------
;
DEBUG = 0
OS_TYPE = !VERSION.OS_FAMILY
;
if keyword_set(no_print)     then do_print=0    else do_print=1
if keyword_set(descript_str) then do_descript=1 else do_descript=0
if keyword_set(waves)        then do_waves=1    else do_waves=0
;
case OS_TYPE of
  'Windows': begin
    do_byteorder=1
  end
  'unix':begin
    if(!VERSION.OS eq 'linux') then do_byteorder=1 else do_byteorder=0
  end
endcase
;
; THERE ARE 4 TYPES OF RECORDS
;
; ITYPE = 0 ; 1st record of data
; ITYPE = 1 ; 1st record of text
; ITYPE = 2 ; continuation record of data
; ITYPE = 3 ; continuation record of text
;
if(do_print eq 1) then begin
  print,'Starting read_specpr.pro'
  if(do_byteorder eq 0) then begin
    print,'Operating System Type = ',OS_TYPE
    print,'Performing byteorder operations.'
  endif else begin
    print,'Operating System Type = ',OS_TYPE
    print,'NOT performing byteorder operations.'
  endelse
endif
;
record_header_text   = {$
    itype:1b,$
    icflag_bit0_contin:0b,$
    icflag_bit1_data_or_text:1b,$
    icflag_bit2_error:0b,$
    icflag_bit3_radec_or_latlon:0b,$
    icflag_bit4_iscta_univ_or_civil:0b,$
    icflag_bit5_isctb_univ_or_civil:0b,$
    ititl:' ',$
    usernm:' ',$
    txtpt:0L,$
    txtch:0L}
;
; NOTES: THESE ARE DEVIATIONS OF THIS PROGRAM'S OUTPUT HEADER FROM STORED FORMAT IN SPECPR RECORDS
;
;  iscta = Civil or Universal time at the beginning of the spectral run.
;          Scaled by 24000.  (see icflag_bit4_iscta_univ_or_civil; 0=UniversalTime 1=CivilTime)
;  isctb = Civil or Universal time at the end of the spectral run.
;          Scaled by 24000.  (see icflag_bit4_iscta_univ_or_civil; 0=UniversalTime 1=CivilTime)
;
SCALE_TIME = 24000.
;
;  jdatea = Date when data was last processed.  Stored as integer*4 Julian Day *10.
;  jdateb = Date when spectral run began.  Stored as integer*4 Julian Day *10.
;
SCALE_JDAY = 10.
;
record_header_data = {$
    itype:0b,$
    icflag_bit0_contin:0b,$
    icflag_bit1_data_or_text:0b,$
    icflag_bit2_error:0b,$
    icflag_bit3_radec_or_latlon:0b,$
    icflag_bit4_iscta_univ_or_civil:0b,$
    icflag_bit5_isctb_univ_or_civil:0b,$
    ititl:' ',$
    usernm:' ',$
    iscta:0.,$
    isctb:0.,$
    jdatea:0.,$
    jdateb:0.,$
    istb:0L,$
    isra:0L,$
    isdec:0L,$
    itchan:0L,$
    irmas:0L,$
    revs:0L,$
    iband:lonarr(2),$
    irwav:0L,$
    irespt:0L,$
    irecno:0L,$
    itpntr:0L,$
    ihist:' ',$
    mhist:strarr(4),$
    nruns:0L,$
    siangl:0L,$
    seangl:0L,$
    sphase:0L,$
    iwtrns:0L,$
    itimch:0L,$
    xnrm:0.0,$
    scatin:0.0,$
    timint:0.0,$
    tempd:0.0$
    }
;
LENGTH_OF_RECORD         = 1536
LENGTH_OF_TEXT_1ST_REC   = 1476
LENGTH_OF_TEXT_ADDTL_REC = 1532
LENGTH_OF_ADDTL_REC      = 1532
NUM_CHANS_PER_REC        =  256
NUM_CHANS_PER_ADDTL_REC  =  383
NUM_BYTES_REC_HEADER     =  512
NUM_TITLE_CHARS          =   40
NUM_USERNM_CHARS         =    8
NUM_IHIST_CHARS          =   60
NUM_MHIST_CHARS          =   74
BAD_DATA_VALUE           = -1.23e+34
;
temp_header        = bytarr(NUM_BYTES_REC_HEADER)
temp_byte1         = 0B
temp_byte2         = 0B
temp_byte3         = 0B
temp_byte4         = 0B
temp_long          = 0L
temp_long2         = replicate(0L,2)
temp_title         = bytarr(NUM_TITLE_CHARS)
temp_username      = bytarr(NUM_USERNM_CHARS)
temp_ihist         = bytarr(NUM_IHIST_CHARS)
temp_mhist_line    = bytarr(NUM_MHIST_CHARS)
temp_float         = 0.0
temp_data          = fltarr(NUM_CHANS_PER_REC)
temp_addtl_data    = fltarr(NUM_CHANS_PER_ADDTL_REC)
temp_text_1strec   = bytarr(LENGTH_OF_TEXT_1ST_REC)
temp_text_addtlrec = bytarr(LENGTH_OF_TEXT_ADDTL_REC)
;
; CHECK THE RECORD NUMBER
;
if(irecord le 0) then begin
  print,'Error: Record number must be greater than 0.'
  return,-1
endif
;
if(DEBUG ge 1) then print,'Starting to read the header...'
;
; OPEN THE SPECPR FILE AND CHECK IF IT IS A VALID FILE
;  If err is nonzero, something happened. Print the error message to
;  the standard error file (logical unit -2):
;
test_size = check_specpr_file_size(specprfile,FILE_SIZE=file_size)
if (test_size ne 1) then begin
  str1='Error: The selected file does not have correct file size'
  str2='       to be a SPECPR file (size must be a multiple of 1536 bytes).'
  str3='       Selected file = '+specprfile
  str4='       File size = '+strcompress(string(file_size),/REMOVE_ALL)
  print,str1,str2,str3,str4
  return,-2
endif
openr,lun_specpr,specprfile,ERROR=err,/GET_LUN
if (err ne 0) then begin
  print,'Error: Could not open selected input file.'
  print,' Error message = ',!ERROR_STATE.MSG
  return,-2
endif
;
; POSITION THE FILE POINTER TO THE SELECTED RECORD (NOTE:THIS POINTS PAST THE 1536 BYTE  HEADER ON THE SPECPR FILE)
;
pointer_position = long64(LENGTH_OF_RECORD)*long64(irecord)
point_lun,lun_specpr,pointer_position
if(DEBUG ge 1) then print,'Pointer position = ',pointer_position,' (record = ',irecord,' where each record = ',LENGTH_OF_RECORD,' bytes).'
;
; CHECK TO SEE IF THE REQUESTED RECORD IS BEYOND THE EOF
;
if (eof(lun_specpr) eq 1) then begin
  close,lun_specpr
  free_lun,lun_specpr
  if(do_print eq 1) then begin
    print,' '
    print,'Requested record is beyond the end of the file!'
    print,' '
  endif
  return,-1
endif
;
; READ 32-BITS, THE FIRST 5 BITS CONTAINS INFO ON THE FILE
;
readu,lun_specpr,temp_byte1
first8bits  = decompose_bits(temp_byte1)
readu,lun_specpr,temp_byte2
second8bits = decompose_bits(temp_byte2)
readu,lun_specpr,temp_byte3
third8bits  = decompose_bits(temp_byte3)
readu,lun_specpr,temp_byte4
fourth8bits = decompose_bits(temp_byte4)
;
if(DEBUG ge 1) then begin
  print,'For the 1st byte = ',temp_byte1,' the bits =  ',first8bits
  print,'For the 2nd byte = ',temp_byte2,' the bits =  ',second8bits
  print,'For the 3rd byte = ',temp_byte3,' the bits =  ',third8bits
  print,'For the 4th byte = ',temp_byte4,' the bits =  ',fourth8bits
endif
;
; SET THE BIT FLAGS IN THE HEADER
;
record_header_data.icflag_bit0_contin              = fourth8bits(0)
record_header_data.icflag_bit1_data_or_text        = fourth8bits(1)
record_header_data.icflag_bit2_error               = fourth8bits(2)
record_header_data.icflag_bit3_radec_or_latlon     = fourth8bits(3)
record_header_data.icflag_bit4_iscta_univ_or_civil = fourth8bits(4)
record_header_data.icflag_bit5_isctb_univ_or_civil = fourth8bits(5)
;
if(record_header_data.icflag_bit1_data_or_text eq 0) then begin
  if(record_header_data.icflag_bit0_contin eq 0) then begin
    itype = 0
  endif else begin
    itype = 2
  endelse
endif else begin
  record_header_text.icflag_bit0_contin              = fourth8bits(0)
  record_header_text.icflag_bit1_data_or_text        = fourth8bits(1)
  record_header_text.icflag_bit2_error               = fourth8bits(2)
  record_header_text.icflag_bit3_radec_or_latlon     = fourth8bits(3)
  record_header_text.icflag_bit4_iscta_univ_or_civil = fourth8bits(4)
  record_header_text.icflag_bit5_isctb_univ_or_civil = fourth8bits(5)
  if(record_header_text.icflag_bit0_contin eq 0) then begin
    itype = 1
  endif else begin
    itype = 3
  endelse
endelse
;
;  SET THE RECORD TYPE
;
if(ITYPE eq 1) then record_header_text.itype = itype else record_header_data.itype = itype
;
;  FOR A FIRST RECORD OF DATA/TEXT READ THE COMMON INFORMATION
;  *** THE FOLLOWING ASSUMES THE REQUESTED RECORD IS THE 1st RECORD          ***
;  *** ALTER CODE TO CHECK IF THE REQUESTED RECORD WAS A CONTINUATION RECORD ***
;
readu,lun_specpr,temp_title
record_header_data.ititl = string(temp_title)
title = record_header_data.ititl
;
readu,lun_specpr,temp_username
record_header_data.usernm = string(temp_username)
;
readu,lun_specpr,temp_long
if(do_byteorder eq 1) then byteorder,temp_long,/LSWAP
record_header_data.iscta = temp_long
;
readu,lun_specpr,temp_long
if(do_byteorder eq 1) then byteorder,temp_long,/LSWAP
record_header_data.isctb = temp_long
;
if(ITYPE eq 0) then begin
  record_header_data.iscta = float(temp_long)/SCALE_TIME
  test_convert = Convert_Seconds_To_HourMinSec(record_header_data.iscta,HOURS=iscta_hours,MINUTES=iscta_minutes,SECONDS=iscta_seconds)
  record_header_data.isctb = float(temp_long)/SCALE_TIME
  test_convert = Convert_Seconds_To_HourMinSec(record_header_data.isctb,HOURS=isctb_hours,MINUTES=isctb_minutes,SECONDS=isctb_seconds)
  if (DEBUG ge 1) then begin
    print,'HEADER INFORMATION:'
    print,' icheck_text = ',ITYPE,'  (0=data_1strec, 1=text_1strec, 2=data_continrec, 3=text_continrec)'
    print,' Title                       = ',record_header_data.ititl
    print,' Title in bytes              = ',temp_title
    print,' User_Name                   = ',record_header_data.usernm
    print,'Username in bytes            = ',temp_username
    print,' Time_Begin_of_Spectral_Run  = ',record_header_data.iscta,' seconds.'
    print,'                             = ',strcompress(string(iscta_hours),/REMOVE_ALL),',',$
                                            strcompress(string(iscta_minutes),/REMOVE_ALL),',',$
                                            strcompress(string(iscta_seconds),/REMOVE_ALL),'(Hours,Minutes,Seconds).'
    print,' Time_End_of_Spectral_Run    = ',record_header_data.isctb,' seconds.'
    print,'                             = ',strcompress(string(isctb_hours),/REMOVE_ALL),',',$
                                            strcompress(string(isctb_minutes),/REMOVE_ALL),',',$
                                            strcompress(string(isctb_seconds),/REMOVE_ALL),'(Hours,Minutes,Seconds).'
  endif
endif else begin
  record_header_text.ititl  = record_header_data.ititl
  record_header_text.usernm = record_header_data.usernm
  record_header_text.txtpt  = record_header_data.iscta
  record_header_text.txtch  = record_header_data.isctb
  if (DEBUG ge 1) then begin
    print,'HEADER INFORMATION:'
    print,' icheck_text = ',record_header_text.ititl,'  (0=data_1strec, 1=text_1strec, 2=data_continrec, 3=text_continrec)'
    print,' Title                        = ',record_header_text.ititl
    print,' User_Name                    = ',record_header_text.usernm
    print,' Text_Data_Record_Pointer     = ',record_header_text.txtpt
    print,' Number_of_Characters_of_Text = ',record_header_text.txtch
  endif
endelse
;
; IF A TEXT RECORD READ TEXT
;
if(ITYPE eq 1) then begin
  num_chars_in_text_rec = record_header_text.txtch
  num_chars_more = num_chars_in_text_rec - LENGTH_OF_TEXT_1ST_REC
  num_addtl_text_recs = ceil(float(num_chars_in_text_rec - LENGTH_OF_TEXT_1ST_REC)/float(LENGTH_OF_TEXT_ADDTL_REC))
  num_recs = num_addtl_text_recs+1
  if(do_print eq 1) then print,'Normal termination of read_specpr.pro'
  close,lun_specpr
  free_lun,lun_specpr
  record_header=record_header_text
  return,1
endif else begin
;
; IF A DATA RECORD READ PARAMATERS AND DATA
;
  readu,lun_specpr,temp_long
  if(do_byteorder eq 1) then byteorder,temp_long,/LSWAP
  record_header_data.jdatea = float(temp_long)/SCALE_JDAY
  CALDAT,record_header_data.jdatea,jdatea_month,jdatea_day,jdatea_year
;
  readu,lun_specpr,temp_long
  if(do_byteorder eq 1) then byteorder,temp_long,/LSWAP
  record_header_data.jdateb = float(temp_long)/SCALE_JDAY
  CALDAT,record_header_data.jdateb,jdateb_month,jdateb_day,jdateb_year
;
  readu,lun_specpr,temp_long
  if(do_byteorder eq 1) then byteorder,temp_long,/LSWAP
  record_header_data.istb = temp_long
;
  readu,lun_specpr,temp_long
  if(do_byteorder eq 1) then byteorder,temp_long,/LSWAP
  record_header_data.isra = temp_long
;
  readu,lun_specpr,temp_long
  if(do_byteorder eq 1) then byteorder,temp_long,/LSWAP
  record_header_data.isdec = temp_long
;
  readu,lun_specpr,temp_long
  if(do_byteorder eq 1) then byteorder,temp_long,/LSWAP
  record_header_data.itchan = temp_long
;
  readu,lun_specpr,temp_long
  if(do_byteorder eq 1) then byteorder,temp_long,/LSWAP
  record_header_data.irmas = temp_long
;
  readu,lun_specpr,temp_long
  if(do_byteorder eq 1) then byteorder,temp_long,/LSWAP
  record_header_data.revs = temp_long
;
  readu,lun_specpr,temp_long2
  if(do_byteorder eq 1) then byteorder,temp_long2,/LSWAP
  record_header_data.iband = temp_long2
;
  readu,lun_specpr,temp_long
  if(do_byteorder eq 1) then byteorder,temp_long,/LSWAP
  record_header_data.irwav = temp_long
;
  readu,lun_specpr,temp_long
  if(do_byteorder eq 1) then byteorder,temp_long,/LSWAP
  record_header_data.irespt = temp_long
;
  readu,lun_specpr,temp_long
  if(do_byteorder eq 1) then byteorder,temp_long,/LSWAP
  record_header_data.irecno = temp_long
;
  readu,lun_specpr,temp_long
  if(do_byteorder eq 1) then byteorder,temp_long,/LSWAP
  record_header_data.itpntr = temp_long
;
  readu,lun_specpr,temp_ihist
  record_header_data.ihist = string(temp_ihist)
;
  readu,lun_specpr,temp_mhist_line
  record_header_data.mhist(0) = string(temp_mhist_line)
;
  readu,lun_specpr,temp_mhist_line
  record_header_data.mhist(1) = string(temp_mhist_line)
;
  readu,lun_specpr,temp_mhist_line
  record_header_data.mhist(2) = string(temp_mhist_line)
;
  readu,lun_specpr,temp_mhist_line
  record_header_data.mhist(3) = string(temp_mhist_line)
;
  readu,lun_specpr,temp_long
  if(do_byteorder eq 1) then byteorder,temp_long,/LSWAP
  record_header_data.nruns = temp_long
;
  readu,lun_specpr,temp_long
  if(do_byteorder eq 1) then byteorder,temp_long,/LSWAP
  record_header_data.siangl = temp_long
;
  readu,lun_specpr,temp_long
  if(do_byteorder eq 1) then byteorder,temp_long,/LSWAP
  record_header_data.seangl = temp_long
;
  readu,lun_specpr,temp_long
  if(do_byteorder eq 1) then byteorder,temp_long,/LSWAP
  record_header_data.sphase = temp_long
;
  readu,lun_specpr,temp_long
  if(do_byteorder eq 1) then byteorder,temp_long,/LSWAP
  record_header_data.iwtrns = temp_long
;
  readu,lun_specpr,temp_long
  if(do_byteorder eq 1) then byteorder,temp_long,/LSWAP
  record_header_data.itimch = temp_long
;
  readu,lun_specpr,temp_float
  if(do_byteorder eq 1) then byteorder,temp_float,/LSWAP
  record_header_data.xnrm = temp_float
;
  readu,lun_specpr,temp_float
  if(do_byteorder eq 1) then byteorder,temp_float,/LSWAP
  record_header_data.scatin = temp_float
;
  readu,lun_specpr,temp_float
  if(do_byteorder eq 1) then byteorder,temp_float,/LSWAP
  record_header_data.timint = temp_float
;
  readu,lun_specpr,temp_float
  if(do_byteorder eq 1) then byteorder,temp_float,/LSWAP
  record_header_data.tempd = temp_float
;
  if(do_print eq 1) then begin
    print,'Reading specprfile (',specprfile,') record =',irecord
    print,'  itype = ',record_header_data.itype
    print,'  Title = ',record_header_data.ititl
    print,'  Time_Begin_of_Spectral_Run = ',record_header_data.iscta,' seconds.'
    print,'                             = ',strcompress(string(iscta_hours),/REMOVE_ALL),',',$
                                            strcompress(string(iscta_minutes),/REMOVE_ALL),',',$
                                            strcompress(string(iscta_seconds),/REMOVE_ALL),'(Hours,Minutes,Seconds).'
    print,'  Time_End_of_Spectral_Run   = ',record_header_data.isctb,' seconds.'
    print,'                             = ',strcompress(string(isctb_hours),/REMOVE_ALL),',',$
                                            strcompress(string(isctb_minutes),/REMOVE_ALL),',',$
                                            strcompress(string(isctb_seconds),/REMOVE_ALL),'(Hours,Minutes,Seconds).'
    print,'  Date last processed        = ',record_header_data.jdatea,' Julian Day'
    print,'                             = ',strcompress(string(jdatea_month),/REMOVE_ALL),'/',$
                                            strcompress(string(jdatea_day),/REMOVE_ALL),'/',$
                                            strcompress(string(jdatea_year),/REMOVE_ALL),' (Month/Day/Year)'
    print,'  Date spectral run began    = ',record_header_data.jdateb,' Julian Day'
    print,'                             = ',strcompress(string(jdateb_month),/REMOVE_ALL),'/',$
                                            strcompress(string(jdateb_day),/REMOVE_ALL),'/',$
                                            strcompress(string(jdateb_year),/REMOVE_ALL),' (Month/Day/Year)'
    print,'  # of Channels    = ',record_header_data.itchan
    print,'  Auto-History     = ',record_header_data.ihist
    print,'  Manual-History-1 = ',record_header_data.mhist(0)
    print,'  Manual-History-2 = ',record_header_data.mhist(1)
    print,'  Manual-History-3 = ',record_header_data.mhist(2)
    print,'  Manual-History-4 = ',record_header_data.mhist(3)
  endif
;
  chans_data_more = record_header_data.itchan - NUM_CHANS_PER_REC
  if(DEBUG ge 1) then print,'Number of channels of data = ',record_header_data.itchan
  if(chans_data_more le 0) then begin
    num_recs_more = 0
  endif else begin
    num_recs_more = ceil(float(chans_data_more)/float(NUM_CHANS_PER_ADDTL_REC))
    if(DEBUG ge 1) then print,'Number of additional records to read = ',num_recs_more
  endelse
;
  if(do_print eq 1) then print,'Done reading the header.'
;
  close,lun_specpr
  free_lun,lun_specpr
;
  if (DEBUG ge 1) then begin
    print,'HEADER INFORMATION:'
    print,' icflag_bit0_contin               = ',record_header_data.icflag_bit0_contin
    print,' icflag_bit1_data_or_text         = ',record_header_data.icflag_bit1_data_or_text
    print,' icflag_bit2_error                = ',record_header_data.icflag_bit2_error
    print,' icflag_bit3_radec_or_latlon      = ',record_header_data.icflag_bit3_radec_or_latlon
    print,' icflag_bit4_iscta_univ_or_civil  = ',record_header_data.icflag_bit4_iscta_univ_or_civil
    print,' icflag_bit5_isctb_univ_or_civil  = ',record_header_data.icflag_bit5_isctb_univ_or_civil
    print,' ititl  = ',record_header_data.ititl
    print,' usernm = ',record_header_data.usernm
    print,' iscta  = ',record_header_data.iscta
    print,' isctb  = ',record_header_data.isctb
    print,' jdatea = ',record_header_data.jdatea
    print,' jdateb = ',record_header_data.jdateb
    print,' istb   = ',record_header_data.istb
    print,' isra   = ',record_header_data.isra
    print,' isdec  = ',record_header_data.isdec
    print,' itchan = ',record_header_data.itchan
    print,' irmas  = ',record_header_data.irmas
    print,' revs   = ',record_header_data.revs
    print,' irwav  = ',record_header_data.irwav
    print,' irespt = ',record_header_data.irespt
    print,' irecno = ',record_header_data.irecno
    print,' itpntr = ',record_header_data.itpntr
    print,' ihist  = ',record_header_data.ihist
    print,' mhist-1= ',record_header_data.mhist(0)
    print,' mhist-2= ',record_header_data.mhist(1)
    print,' mhist-3= ',record_header_data.mhist(2)
    print,' mhist-4= ',record_header_data.mhist(3)
    print,' nruns  = ',record_header_data.nruns
    print,' siangl = ',record_header_data.siangl
    print,' seangl = ',record_header_data.seangl
    print,' sphase = ',record_header_data.sphase
    print,' iwtrns = ',record_header_data.iwtrns
    print,' itimch = ',record_header_data.itimch
    print,' xnrm   = ',record_header_data.xnrm
    print,' scatin = ',record_header_data.scatin
    print,' timint = ',record_header_data.timint
    print,' tempd  = ',record_header_data.tempd
  endif
;
; SET THE NUMBER OF RECORDS THAT THIS ENTRY USES FOR RETURN TO COMMAND LINE
  num_recs = num_recs_more + 1
;
  if(do_print eq 1) then print,'Normal termination of read_specpr.pro'
  close,lun_specpr
  free_lun,lun_specpr
  record_header=record_header_data
  return,0
;
endelse  ;END ELSE FOR READING OF DATA RECORDS
;
end